*! version 1.6.0 - 11/19/00
capture program drop praccum
program define praccum
    version 6
    tempname newmat
*=> classify each valid type of model
    if "`e(cmd)'"=="cloglog"  { local io = "typical binary" }
    if "`e(cmd)'"=="cnreg"    { local io = "typical regress" }
    if "`e(cmd)'"=="fit"      { local io = "typical regress" }
    if "`e(cmd)'"=="gologit"  { local io = "typical mlogit" }
    if "`e(cmd)'"=="intreg"   { local io = "typical regress" }
    if "`e(cmd)'"=="logistic" { local io = "typical none" }
    if "`e(cmd)'"=="logit"    { local io = "typical binary" }
    if "`e(cmd)'"=="mlogit"   { local io = "typical mlogit" }
    if "`e(cmd)'"=="nbreg"    { local io = "typical count" }
    if "`e(cmd)'"=="ologit"   { local io = "typical ordered" }
    if "`e(cmd)'"=="oprobit"  { local io = "typical ordered" }
    if "`e(cmd)'"=="poisson"  { local io = "typical count" }
    if "`e(cmd)'"=="probit"   { local io = "typical binary" }
    if "`e(cmd)'"=="regress"  { local io = "typical regress" }
    if "`e(cmd)'"=="tobit"    { local io = "typical regress" }
    if "`e(cmd)'"=="zinb"     { local io = "twoeq count" }
    if "`e(cmd)'"=="zip"      { local io = "twoeq count" }
    if "`io'"=="" {
        di
        di in y "prvalue" in r /*
        */ " does not work for the last type of model estimated."
        exit
    }
    local input : word 1 of `io'   /* input routine to _pepred */
    local output : word 2 of `io'  /* output routine */

*=> decode specified input
    syntax [, Saving(string) Using(string) GENerate(string) XIS(string)]

*truncate generate root if more than five characters
    if "`generate'" ~= "" {
        local gen = substr("`generate'",1,5)
    }

        tempname results
        if "`output'" == "regress" {
            matrix `results' = ( `xis' , r(xb) )
        }
        if "`output'" == "binary" {
            * grab output from binary model
            matrix `results' = ( `xis' , r(p0) , r(p1) )
        }
        if "`output'" == "ordered" | "`output'" == "mlogit" | /*
            */ "`output'" == "count" {
            tempname probs newprob
            mat `probs' = r(probs)
            local outcms = rowsof(r(probs))
            matrix `results' = `xis'
            local count = 1
            while `count' <= `outcms' {
                matrix `newprob' = `probs'[`count', 1]
                * grab output from binary model
                matrix `results' = `results' , `newprob'
                local count = `count' + 1
            }
        }
        if "`input'" == "twoeq" & "`output'"=="count" {
            tempname az
            matrix `az' = r(always0)
            matrix `results' = `results' , `az'
        }

*=> saving is the initial run
    if "`saving'" ~= "" { mat `saving' = `results' }
    if "`using'" ~= "" {
        qui mat list `using'
        if _rc ~= 0 {
            di in r "matrix `using' does not exist"
            exit 111
        }
        mat `using' = (`using') \ (`results')
    }

*=> generates creates the new variables
    if "`gen'" ~= "" {
        if "`output'" == "regress" {
            local columns = "`gen'x `gen'xb"
        }
        if "`output'" == "binary" {
            local columns = "`gen'x `gen'p0 `gen'p1"
        }
        if "`output'" == "ordered" | "`output'" == "mlogit" {
            local columns "`gen'x"
            local outcms = rowsof(r(probs))
            local count = 1
            while `count' <= `outcms' {
                local columns "`columns' `gen'p`count'"
                local count = `count' + 1
            }
        }
        if "`output'" == "count" {
            local columns "`gen'x"
            local outcms = rowsof(r(probs))
            local count = 0
            while `count' <= (`outcms'-1) {
                local columns "`columns' `gen'p`count'"
                local count = `count' + 1
            }
            if "`input'"=="twoeq" {
                local columns "`columns' `gen'az"
            }
        }
        * create new variables
        matrix colnames `using' = `columns'
        svmat `using', names(col)

    *=> label variables
        label variable `gen'x "value of x"
        if "`output'" == "regress" {
            label variable `gen'xb "value of xb"
        }
        if "`output'" == "binary" {
            label variable `gen'p0 "Pr(0)"
            label variable `gen'p1 "Pr(1)"
        }
        if "`output'" == "ordered" | "`output'" == "mlogit" | "`output'" == "count" {
            tempname values
            mat `values' = r(values)
            local outcms = rowsof(r(probs))
            local count = 1
            while `count' <= `outcms' {
                local value = `values'[`count', 1]
                local count2 = `count'
                if "`output'"=="count" { local count2 = `count' - 1 }
                label variable `gen'p`count2' "Pr(`value')"
                local count = `count' + 1
            }
            if "`input'"=="twoeq" {
                label variable `gen'az "Pr(always0)"
            }
        }

*=> for ordered and count variables, generate cumulative counts
        if "`output'" == "ordered" {
            local outcms = rowsof(r(probs))
            local count = 1
            while `count' <= `outcms' {
                qui egen `gen's`count' = rsum(`gen'p1-`gen'p`count') if `gen'p1~=.
                local cumul = "`cumul'`gen's`count' "
                label variable `gen's`count' "Pr(<=`count')"
                local count = `count' + 1
            }
        }
        if "`output'" == "count" {
            local outcms = rowsof(r(probs))
            local count = 0
            while `count' <= (`outcms'-1) {
                qui egen `gen's`count' = rsum(`gen'p0-`gen'p`count') if `gen'p0~=.
                local cumul = "`cumul'`gen's`count' "
                label variable `gen's`count' "Pr(<=`count')"
                local count = `count' + 1
            }
        }

    if "`output'" == "ordered" | "`output'" == "mlogit" {
        di _n in blu "warning: numbers in variable names might not correspond to category values."
        di in blu "correct category values are provided in the new variable labels."
    }

*=> display new variables
        di _n in g "New variables created by" in w " praccum" in y ":"
        sum `columns' `cumul'
    } /* generate */
end

